package com.gic.haoban.manage.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.DataApiUtils;
import com.gic.commons.util.DateUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreSearchDTO;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.web.qo.wechatwork.StoreCommonQO;
import com.gic.haoban.manage.web.qo.wechatwork.WechatWorkLostOverviewQO;
import com.gic.haoban.manage.web.utils.data.ConcurrencyUtils;
import com.gic.haoban.manage.web.utils.data.MapThreadHandlerRequest;
import com.gic.haoban.manage.web.vo.wechatwork.WechatWorkLostVO;
import com.gic.web.common.controller.NewBaseController;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 企微流失接口
 * @Author guojx
 * @Date 2024/5/21 13:45
 */
@RestController
@Slf4j
@RequestMapping("wechat-work")
public class WechatWorkController extends NewBaseController {
    @Autowired
    private StoreGroupService storeGroupService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private StoreService storeService;

    /**
     * 流失数据概览
     * @param qo
     * @return
     */
    @RequestMapping(value = "lost-overview")
    public RestResponse<WechatWorkLostVO> lostOverview(@RequestBody WechatWorkLostOverviewQO qo) {
        JSONObject jsonObject = (JSONObject) JSONObject.toJSON(qo);
        setCommonParam(jsonObject, qo);
        Map<String, Object> res = DataApiUtils.http(jsonObject.toJSONString(), "data_mbr_scale_haoban_background_loss");
        List<JSONObject> list = DataApiUtils.getDataList(res);
        List<WechatWorkLostVO> voList = JSONArray.parseArray(JSON.toJSONString(list), WechatWorkLostVO.class);
        return RestResponse.successResult(CollectionUtils.isEmpty(voList) ? null : voList.get(0));
    }

    /**
     * 流失数据概览趋势图
     * @param qo
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "lost-overview-trend")
    public RestResponse<List<WechatWorkLostVO>> lostOverviewTrend(@RequestBody WechatWorkLostOverviewQO qo) throws ParseException {
        JSONObject jsonObject = (JSONObject) JSONObject.toJSON(qo);
        setCommonParam(jsonObject, qo);

        List<MapThreadHandlerRequest> list = new ArrayList<>();
        jsonObject.put("groupType", 1);
        list.add(new MapThreadHandlerRequest(jsonObject, "data_mbr_scale_haoban_background_loss_date", "1_onewayFriendNum"));
        jsonObject.put("groupType", 2);
        list.add(new MapThreadHandlerRequest(jsonObject, "data_mbr_scale_haoban_background_loss_date", "2_totalLostNum"));
        jsonObject.put("groupType", 3);
        list.add(new MapThreadHandlerRequest(jsonObject, "data_mbr_scale_haoban_background_loss_date", "3_deleteSalesNum"));
        jsonObject.put("groupType", 4);
        list.add(new MapThreadHandlerRequest(jsonObject, "data_mbr_scale_haoban_background_loss_date", "4_salesDeleteNum"));

        Map<String, List<JSONObject>> map = ConcurrencyUtils.concurrencyDataForMap(list);
        List<JSONObject> resultList = new ArrayList<>();
        if (map != null) {
            map.forEach((k, v) -> {
                if (CollectionUtils.isNotEmpty(v)) {
                    for (JSONObject tempJson : v) {
                        tempJson.put(k.substring(0, 1), tempJson.get(k.substring(2)));
                    }
                    resultList.addAll(v);
                }
            });
        }
        Map<String, List<JSONObject>> voMap = resultList.stream().collect(Collectors.groupingBy(e -> e.getString("bizDate")));

        List<String> dateList = DateUtil.getBetweenDates(qo.getStartDate(), qo.getEndDate());
        List<WechatWorkLostVO> voList = new ArrayList<>();
        for (String date : dateList) {
            WechatWorkLostVO vo = new WechatWorkLostVO();
            vo.setBizDate(date);
            List<JSONObject> mapValue = voMap.get(date);
            if (CollectionUtils.isNotEmpty(mapValue)) {
                for (JSONObject tempJson : mapValue) {
                    vo.setOnewayFriendNum(tempJson.getInteger("1"));
                    vo.setTotalLostNum(tempJson.getInteger("2"));
                    vo.setDeleteSalesNum(tempJson.getInteger("3"));
                    vo.setSalesDeleteNum(tempJson.getInteger("4"));
                }
            }
            voList.add(vo);
        }
        return RestResponse.successResult(voList);
    }

    private void setCommonParam(JSONObject jsonObject, StoreCommonQO storeCommonQO) {
        List<String> storeIds = getAuthStoreId(storeCommonQO);
        if (CollectionUtils.isNotEmpty(storeIds)) {
            jsonObject.put("storeId", storeIds);
        }
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        jsonObject.put("enterpriseId", login.getEnterpriseId());
    }

    private List<String> getAuthStoreId(StoreCommonQO storeCommonQO) {
        String storeGroupId = storeCommonQO.getStoreGroupId();
        String storeSearchParam = storeCommonQO.getStoreSearchParam();
        String storeId = storeCommonQO.getStoreId();
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        List<String> roleStoreIds = staffApiService.getHaoBanStoreIdsRolesByClerkId(login.getClerkId(), login.getWxEnterpriseId());
        if (CollectionUtils.isEmpty(roleStoreIds)) {
            log.info("登录人无门店权限");
            return getNoExistStore();
        }
        if(StringUtils.isNotBlank(storeGroupId) || StringUtils.isNotBlank(storeSearchParam)) {
            StoreSearchDTO storeSearch = new StoreSearchDTO();
            storeSearch.setEnterpriseId(login.getEnterpriseId());
            if (StringUtils.isNotBlank(storeGroupId)) {
                List<String> storeGroupIds = Arrays.stream(storeGroupId.split(",")).collect(Collectors.toList());
                List<String> groupIds = storeGroupService.getStoreGroupIdsByParentGroupId(login.getEnterpriseId(), storeGroupIds);
                storeSearch.setStoreGroupIdList(groupIds);
            }
            if (StringUtils.isNotBlank(storeSearchParam)) {
                storeSearch.setSearchName(storeSearchParam);
            }
            Page pageStore = new Page<>();
            //-1代表不分页
            pageStore.setCurrentPage(-1);
            Page resultPage = storeService.storeListPage(pageStore, storeSearch);
            List<StoreDTO> storeDTOList = resultPage.getResult();
            if (CollectionUtils.isEmpty(storeDTOList)) {
                return getNoExistStore();
            }
            List<String> searchStoreIdList = storeDTOList.stream().map(e -> e.getStoreId()).collect(Collectors.toList());
            if (isAllAuth(roleStoreIds)) {
                return searchStoreIdList;
            }
            //交集
            roleStoreIds.retainAll(searchStoreIdList);
            if (CollectionUtils.isEmpty(roleStoreIds)) {
                return getNoExistStore();
            }
        }
        if (StringUtils.isNotBlank(storeId)) {
            List<String> ids = Arrays.stream(storeId.split(",")).collect(Collectors.toList());
            roleStoreIds.retainAll(ids);
            if (CollectionUtils.isEmpty(roleStoreIds)) {
                return getNoExistStore();
            }
        }
        return roleStoreIds;
    }

    private static boolean isAllAuth(List<String> roleStoreIds) {
        return roleStoreIds.size() == 1 && "-1".equals(roleStoreIds.get(0));
    }

    private static List<String> getNoExistStore() {
        List<String> list = new ArrayList<>(1);
        list.add("no_exist_store_id");
        return list;
    }

}
